@@ -24,7 +24,7 @@ Readonly my $MAXVAL_16_BIT  => 2**$_16_BIT - 1;
 my $uuid_object = Data::UUID->new;
 my $EMPTY       = q{};
 
-our $VERSION = '1.5.1';
+our $VERSION = '1.5.2';
 
 my ( $prog_name, $logger, %callback, $_self );
 
@@ -842,7 +842,9 @@ sub _write_buffer_to_fh {
     }
     _thread_write_pnm_header( $fh, $parm->{format}, $parm->{pixels_per_line},
         $image->{height}, $parm->{depth} );
-    for ( @{ $image->{data} } ) { goto CLEANUP if not print $fh }
+    for my $data ( @{ $image->{data} } ) {
+        goto CLEANUP if not print {$fh} $data;
+    }
     return;
 }
 
